/********** smtp.c
		SMTP handling routines for the nlist mailing list
		package.
		Written by Gabor Paller
					***************/

#include <stdio.h>
#include <string.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <errno.h>
#include	"config.h"
#define		N_EXTERN	extern
#include	"commonvars.h"

/* SMTP IANA number */
#define SMTP_PORT		25

static int	sock;		/* socket fd */ 

/* Opens a socket to a given port of the given server. Returns 0
   if OK, error code else */
int open_socket( char server[],int port ) {
   struct sockaddr_in server_addr;
   struct hostent *hp, *gethostbyname();
   char buf[1024];
   
/* Create socket */
   sock = socket( AF_INET, SOCK_STREAM, 0 );
   if (sock == -1) {
     Log( 3,1,"Could not create stream socket: %s",strerror( errno ) );
     return 1;
   }
	
/* Connect socket using name specified by command line. */
   server_addr.sin_family = AF_INET;
   hp = gethostbyname( server );
/*
* gethostbyname returns a structure including the network address
* of the specified host
*/
  if (hp == (struct hostent *) 0) {
    Log( 3,1,"Cannot resolve hostname %s: %s",server,strerror( errno ) );
    return 2;
  }
  memcpy((char *) &server_addr.sin_addr, (char *) hp->h_addr,
                   hp->h_length );
  server_addr.sin_port = htons( port );
  if (connect(sock, (struct sockaddr *) &server_addr, sizeof server_addr)
          == -1) {
    Log( 3,1,"Could not connect to %s:%u: %s",server,port,
        		strerror( errno ) );
    return 3;
  }
  return 0; 
}

/* Close the socket connection.
 */
void close_socket() {
  if (close(sock) < 0)
	ehandle( 1,"socket close: %i\n",errno );
}

/* Write out the output buffer to the socket */
void socket_write_line( char *buffer ) {
  char *p = buffer;
  int len;
  int w;

  Log( 4,0,"%s OUT: %s",protstr,buffer );
  len = strlen( buffer );
  while (len > 0) {
    do {
      w = write(sock, p, len);
    } while (w < 0 && errno == EINTR); /* retry on interrupted system call */
    if (w < 0)
	ehandle( 1,"socket write: %i\n",errno );
    p += w;			/* write the rest out if short write */
    len -= w;
  }
}

void socket_read_line( char *buffer, int bufsize ) {
  int	i,n;
  char	*p = buffer,*p1;

  do {
    do {
      n = read(sock, p, 1 );
    } while (n < 0 && errno == EINTR);
    if (n < 0)
	ehandle( 1,"socket read: %i\n",errno );
    if (n == 0) {
      *p = 0;
      Log( 4,0,"%s IN: %s",protstr,buffer );
      return;
    }
    p1 = p;
    ++p;
    --bufsize;
    if( ( *p1 == '\n' ) || ( bufsize == 0 ) ) {
      *p = 0;
      Log( 4,0,"%s IN: %s",protstr,buffer );
      return;
    } 
  } while( 1 );
}

int open_smtp_session( char smtp_server[], char *from, char *to ) {
  char	tempbuffer[256];
  char	outbuffer[256];
  int	retry = 10;

  strcpy( protstr,"SMTP" );
l0:
  if( retry == 0 )
    ehandle( 1,"SMTP error: %i:'%s'\n",errno,tempbuffer );
  if( open_socket( smtp_server, SMTP_PORT ) )
    ehandle( 1,"Couldn't open SMTP connection" );
  sprintf( tempbuffer,"HELO %s\n",domain_name );
  socket_write_line( tempbuffer );
  do {
    socket_read_line( tempbuffer,255 );
    if( strncmp( tempbuffer,"220",3 ) == 0 )	/* Login response */
	continue;
    if( strncmp( tempbuffer,"250",3 ) == 0 )	/* OK */
	break;
/* Some garbage arrived. Terminate the connection,
   close the socket and retry. I don't know how it can happen
   but it happens ... :-( */
    --retry;
    close_socket();
    sleep( 1 );		/* Wait 1 second */
    goto l0;
  } while( 1 );
  sprintf( outbuffer,"MAIL FROM: %s\n",
              subst_lname( from_address,from,to,list_address ) );
  socket_write_line( outbuffer );
  socket_read_line( tempbuffer,255 );
  if( strncmp( tempbuffer,"250",3 ) != 0 )
	return 1;
  sprintf( outbuffer,"RCPT TO: %s\n",to );
  socket_write_line( outbuffer );
  socket_read_line( tempbuffer,255 );
  if( strncmp( tempbuffer,"250",3 ) != 0 )
	return 1;
  socket_write_line( "DATA\n" );
  socket_read_line( tempbuffer,255 );
  if( strncmp( tempbuffer,"354",3 ) != 0 )
	return 1;
  return 0;
}

void write_smtp_line( char *buffer ) {
  if( strncmp( buffer,".\n",2 ) == 0 )
	socket_write_line( " .\n" );
  else
	socket_write_line( buffer ); 
}

void close_smtp_session() {
  socket_write_line( "\n.\n" );
  close_socket();
}
